
$.loadText = function (file_path, callback) {
    if (window.TYRANO) window.TYRANO.kag.showLoadingLog();

    $.ajax({
        url: file_path + "?" + Math.floor(Math.random() * 1000000),
        dataType: 'text',
        cache: false,
        success: function (text) {
            if (window.TYRANO) window.TYRANO.kag.hideLoadingLog();
            const order_str = text;
            callback(order_str);
        },
        error: function () {
            if (window.TYRANO) window.TYRANO.kag.hideLoadingLog();
            alert($.lang("file_not_found", { path: file_path }));
            callback("");
        },
    });
};

$.loadTextSync = function (file_path) {
    return new Promise((resolve, reject) => {
        $.ajax({
            url: file_path + "?" + Math.floor(Math.random() * 1000000),
            dataType: 'text',
            cache: false,
            success: function (text) {
                if (window.TYRANO) window.TYRANO.kag.hideLoadingLog();
                const order_str = text;
                resolve(order_str);
            },

            error: function () {
                if (window.TYRANO) window.TYRANO.kag.hideLoadingLog();
                alert($.lang("file_not_found", { path: file_path }));
                reject();
            },
        });

    });
};


TYRANO.kag.parser.parseScenario = (text_str) => {

    this.kag = TYRANO.kag;

    var array_s = [];

    var map_label = {}; //ラベル一覧

    var array_row = text_str.split("\n");

    var flag_comment = false; //コメント中なら

    for (var i = 0; i < array_row.length; i++) {
        var line_str = $.trim(array_row[i]);
        var first_char = line_str.substr(0, 1);

        if (line_str.indexOf("endscript") != -1) {
            this.kag.parser.flag_script = false;
        }

        //コメントの場合は無視する
        if (flag_comment === true && line_str === "*/") {
            //ブロックコメント解除
            //"*/"単独ではない場合、たとえば"hoge */"とか"*/ hoge"のような行ではブロックコメントは解除されない
            flag_comment = false;
        } else if (line_str === "/*") {
            //ブロックコメント開始
            //やはり"/*"単独の行でないと認識されない
            flag_comment = true;
        } else if (flag_comment == true || first_char === ";") {
            //コメントは無視
        } else if (first_char === "#") {
            //キャラ名
            //#akane:happy
            //↑を↓に変換する
            //[chara_ptext name=akane face=happy]
            var tmp_line = $.trim(line_str.replace("#", ""));
            var chara_name = "";
            var chara_face = "";
            if (tmp_line.split(":").length > 1) {
                var array_line = tmp_line.split(":");
                chara_name = array_line[0];
                chara_face = array_line[1];
            } else {
                chara_name = tmp_line;
            }
            //キャラクターボックスへの名前表示
            var text_obj = {
                line: i,
                name: "chara_ptext",
                pm: { name: chara_name, face: chara_face },
                val: "",
            };

            array_s.push(text_obj);
        } else if (first_char === "*") {
            //ラベル
            //*opening|オープニング
            //↑を↓に変換する
            //[label label_name=opening val=オープニング]
            var label_tmp = line_str.substr(1, line_str.length).split("|");

            var label_key = "";
            var label_val = "";

            label_key = $.trim(label_tmp[0]);

            if (label_tmp.length > 1) {
                label_val = $.trim(label_tmp[1]);
            }

            var label_obj = {
                name: "label",
                pm: {
                    line: i,
                    index: array_s.length,
                    label_name: label_key,
                    val: label_val,
                },
                val: label_val,
            };

            //ラベル
            array_s.push(label_obj);

            if (map_label[label_obj.pm.label_name]) {
                //ラベルの重複はエラー
                this.kag.warning("duplicate_label", { name: label_obj.pm.label_name });
            } else {
                map_label[label_obj.pm.label_name] = label_obj.pm;
            }
        } else if (first_char === "@") {
            //タグ
            //残りの部分をごそっと回す
            const tag_str = line_str.substr(1, line_str.length); // "image split=2 samba = 5"
            const tmpobj = this.kag.parser.makeTag(tag_str, i);
            array_s.push(tmpobj);
        } else {
            //テキストか[]記法のタグ
            //テキストは[iscript]内のJavaScriptや[html]内のHTMLである可能性がある

            //先頭の半角アンダーバーは空白を除去しないという特殊記号なので排除
            if (first_char === "_") {
                line_str = line_str.substring(1, line_str.length);
            }

            //１文字ずつバラして解析していく
            let array_char = line_str.split("");
            let text = ""; // テキスト文字（[iscript][html]内のテキストを含む）
            let tag_str = ""; // タグ文字 例）ptext x=0 y=0 text="hogehoge[][]'''"
            let scanning_tag = false; // タグ解析中かどうか
            let deep_kakko = 0; // [ の深さ
            //↑exp属性の中で配列[]を使用した場合などに、配列の"]"を閉じタグの"]"として解釈しないようにするために必要
            let start_quot = ""; // クォートが始まってから終わるまで
            let flag_escape = false; // エスケープフラグ バックスラッシュでフラグが立つ

            for (var j = 0; j < array_char.length; j++) {
                var c = array_char[j];

                if (this.kag.parser.flag_script) {
                    // [iscript]解析中は単に足す
                    text += c;
                } else if (scanning_tag) {
                    // タグ解析中の場合
                    if (c === "]") {
                        // タグ解析中 → 閉じタグ ] に遭遇した！
                        if (start_quot !== "") {
                            // クォート内部ならどうでもよい
                            // 例) [ptext text="[[あああ]]"] ← こういうのに対応
                            tag_str += c;
                        } else {
                            // 1段表層に浮かび上がる
                            deep_kakko--;
                            if (deep_kakko === 0) {
                                // 最表層に戻ってきたときにタグ文字列が完成する！makeTagに投げる
                                scanning_tag = false;
                                array_s.push(this.kag.parser.makeTag(tag_str, i));
                                tag_str = "";
                                start_quot = "";
                            } else {
                                // まだ最表層ではないなら単に足す
                                // 例) [ptext text=[[あああ]] ] ← こういうのに対応
                                tag_str += c;
                            }
                        }
                    } else if (c === "[") {
                        // タグ解析中 → 開始タグ [ に遭遇した！
                        if (start_quot === "") {
                            // クォート外部であるときだけ1段深層に沈む
                            deep_kakko++;
                        }
                        // 足すの忘れない
                        tag_str += c;
                    } else if (c === '"' || c === "'" || c === "`") {
                        // タグ解析中 → クォート " ' ` に遭遇した！
                        if (start_quot === c) {
                            // このクォートなんか見たことあるぞ…となったらクォートを脱出
                            start_quot = "";
                        } else if (start_quot === "") {
                            // まだクォートに侵入していないならこれから侵入する
                            start_quot = c;
                        }
                        // 足すの忘れない
                        tag_str += c;
                    } else {
                        // タグ解析中 → [ ] " ' ` のいずれでもない
                        tag_str += c;
                    }
                } else if (flag_escape) {
                    // タグ解析中ではなくエスケープフラグが立っている場合
                    text += c;
                    flag_escape = false;
                } else if (c === "[") {
                    // タグ解析中ではなくエスケープフラグも立っていないときに [ に遭遇した場合
                    // タグ解析開始！
                    scanning_tag = true;
                    deep_kakko++;
                    //この時点で格納されているテキストがあれば配列に追加
                    if (text != "") {
                        const text_obj = {
                            line: i,
                            name: "text",
                            pm: { val: text },
                            val: text,
                            is_entity_disabled: true,
                        };
                        array_s.push(text_obj);
                        text = "";
                    }
                } else {
                    //"["以外の文字の場合
                    if (c === "\\") {
                        flag_escape = true;
                    } else {
                        text += c;
                    }
                }
            }
            //　1文字ずつ解析していくのが完了した
            // この時点でタグがあれば配列に追加
            if (tag_str) {
                if (tag_str.slice(-1) === "]") {
                    const tag_str_fixed = tag_str.slice(0, tag_str.length - 1);
                    this.kag.warning(
                        "compensate_missing_quart",
                        {
                            before: tag_str,
                            after: tag_str_fixed,
                        },
                        false,
                    );
                    tag_str = tag_str_fixed;
                }
                array_s.push(this.kag.parser.makeTag(tag_str, i));
            }
            //　この時点でテキストがあれば配列に追加
            if (text) {
                array_s.push({
                    line: i,
                    name: "text",
                    pm: { val: text },
                    val: text,
                    is_entity_disabled: true,
                });
            }

            //console.log(array_char);
        }
        //１行づつ解析解析していく
    }

    //翻訳が必要な場合はここでarray_sの中身を解析して翻訳する
    this.kag.convertLang(this.kag.stat.current_scenario, array_s);

    var result_obj = {
        array_s: array_s,
        map_label: map_label,
    };

    if (this.kag.parser.deep_if != 0) {
        this.kag.error("if_and_endif_do_not_match");
        this.kag.parser.deep_if = 0;
    }

    return result_obj;
};

TYRANO.kag.convertLang = (scenario, array_s) => {

    this.kag = TYRANO.kag;

    if (this.kag.lang == "") return array_s;

    if (!this.kag.map_lang["scenes"][scenario]) {
        return array_s;
    }

    let map_trans = this.kag.map_lang["scenes"][scenario];
    let map_charas = this.kag.map_lang["charas"];


    let is_script = false;

    for (let i = 0; i < array_s.length; i++) {

        const tobj = array_s[i];

        if (tobj.name === "iscript") {
            is_script = true;
        } else if (tobj.name === "endscript") {
            is_script = false;
        } else if (tobj.name === "text") {

            if (!is_script) {

                let trans_text = "";
                if (map_trans["scenario"][tobj.pm.val]) {
                    trans_text = map_trans["scenario"][tobj.pm.val];
                    tobj.pm.val = trans_text;
                    array_s[i] = tobj;
                }

            }

        } else if (tobj.name === "chara_ptext") {

            if (map_charas && map_charas[tobj["pm"]["name"]]) {

                //キャラ名指定の場合はこうなる
                if (this.kag.stat.charas[tobj["pm"]["name"]]) {
                    this.kag.stat.charas[tobj["pm"]["name"]].jname = map_charas[tobj["pm"]["name"]];
                } else {

                    tobj["pm"]["name"] = map_charas[tobj["pm"]["name"]];

                }
                array_s[i] = tobj;
            }

        } else {

            if (map_trans["tag"] && map_trans["tag"][tobj.name]) {

                //翻訳対象のタグだった場合
                let pm = tobj["pm"];
                for (let key in pm) {

                    if (map_trans["tag"][tobj.name][key]) {
                        if (map_trans["tag"][tobj.name][key][pm[key]]) {
                            array_s[i]["pm"][key] = map_trans["tag"][tobj.name][key][pm[key]];
                        }
                    }

                }
            }

        }

    }

    return array_s;

};

TYRANO.kag.loadLang = async (name, cb) => {

    if (name != "default") {

        try {

            let lang_str = await $.loadTextSync("./data/others/lang/" + name + ".json");
            this.kag.lang = name;
            this.kag.map_lang = JSON.parse(lang_str);

        } catch (e) {
            console.log(e);
            this.kag.lang = "";
            this.kag.map_lang = {};
        }

    } else {

        this.kag.lang = "";
        this.kag.map_lang = {};

    }

    //キャッシュは削除
    this.kag.cache_scenario = {};

    this.kag.evalScript("sf._system_config_lang='" + name + "';");

    cb();

};

//
TYRANO.kag.ftag.master_tag.lang_set = {

    vital: ["name"],

    pm: {
        name: "",
    },

    start: (pm) => {

        this.kag = TYRANO.kag;
        //langファイルを読み込んで設定する

        this.kag.loadLang(pm.name, () => {

            const scenario_file = this.kag.stat.current_scenario;

            //呼び出したファイル自身をロード
            this.kag.loadScenario(scenario_file, (array_tag) => {
                this.kag.ftag.array_tag = array_tag;
                this.kag.ftag.nextOrder();
            });

        });


    },
};


